@@ -1,18 +1,7 @@
 /*
-    Copyright (C) 2018 Volker Krause <vkrause@kde.org>
+    SPDX-FileCopyrightText: 2018 Volker Krause <vkrause@kde.org>
 
-    This program is free software; you can redistribute it and/or modify it
-    under the terms of the GNU Library General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or (at your
-    option) any later version.
-
-    This program is distributed in the hope that it will be useful, but WITHOUT
-    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
-    License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <https://www.gnu.org/licenses/>.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "trainstationdbgenerator.h"
@@ -46,23 +35,21 @@ static bool operator<(const TrainStationDbGenerator::Station &lhs, const QUrl &r
 bool TrainStationDbGenerator::generate(QIODevice *out)
 {
     // retrieve content from Wikidata
-    if (!fetchIBNR() || !fetchUIC() || !fetchGaresConnexions() || !fetchIndianRailwaysStationCode()) {
+    if (!fetchIBNR() || !fetchUIC() || !fetchSncf() || !fetchIndianRailwaysStationCode() || !fetchFinishStationCodes()) {
         return false;
     }
     if (!fetchCountryInformation()) {
          return false;
      }
 
-    // timezone lookup and filtering
+    // filtering out stations without useful information
     processStations();
 
     // code generation
     CodeGen::writeLicenseHeader(out);
     out->write(R"(
 #include "knowledgedb.h"
-#include "timezonedb.h"
 #include "trainstationdb.h"
-#include "timezonedb_data_p.h"
 
 namespace KItinerary {
 namespace KnowledgeDb {
@@ -70,8 +57,9 @@ namespace KnowledgeDb {
     writeStationData(out);
     writeIBNRMap(out);
     writeUICMap(out);
-    writeGareConnexionMap(out);
+    writeSncfMap(out);
     writeIndianRailwaysMap(out);
+    writeVRMap(out);
     out->write(R"(
 }
 }
@@ -163,12 +151,12 @@ bool TrainStationDbGenerator::fetchUIC()
     return true;
 }
 
-bool TrainStationDbGenerator::fetchGaresConnexions()
+bool TrainStationDbGenerator::fetchSncf()
 {
     const auto stationArray = WikiData::query(R"(
-        SELECT DISTINCT ?station ?stationLabel ?gareConnexionId ?coord WHERE {
+        SELECT DISTINCT ?station ?stationLabel ?sncfId ?coord WHERE {
             ?station (wdt:P31/wdt:P279*) wd:Q55488.
-            ?station wdt:P3104 ?gareConnexionId.
+            ?station wdt:P8181 ?sncfId.
             OPTIONAL { ?station wdt:P625 ?coord. }
             SERVICE wikibase:label { bd:serviceParam wikibase:language "en". }
         } ORDER BY (?station))", "wikidata_trainstation_gare_connexion.json");
@@ -181,19 +169,19 @@ bool TrainStationDbGenerator::fetchGaresConnexions()
         const auto stationObj = stationData.toObject();
         const auto uri = insertOrMerge(stationObj);
 
-        const auto id = stationObj.value(QLatin1String("gareConnexionId")).toObject().value(QLatin1String("value")).toString().toUpper();
+        const auto id = stationObj.value(QLatin1String("sncfId")).toObject().value(QLatin1String("value")).toString().toUpper();
         if (id.size() != 5 || !Util::containsOnlyLetters(id)) {
             ++m_idFormatViolations;
-            qWarning() << "Gares & Connexions ID format violation" << id << uri;
+            qWarning() << "SNCF ID format violation" << id << uri;
             continue;
         }
 
-        const auto it = m_garesConnexionsIdMap.find(id);
-        if (it != m_garesConnexionsIdMap.end() && (*it).second != uri) {
+        const auto it = m_sncfIdMap.find(id);
+        if (it != m_sncfIdMap.end() && (*it).second != uri) {
             ++m_idConflicts;
-            qWarning() << "Conflict on Gares & Connexions ID" << id << uri << m_garesConnexionsIdMap[id];
+            qWarning() << "Conflict on SNCF ID" << id << uri << m_sncfIdMap[id];
         } else {
-            m_garesConnexionsIdMap[id] = uri;
+            m_sncfIdMap[id] = uri;
         }
     }
 
@@ -231,6 +219,51 @@ bool TrainStationDbGenerator::fetchIndianRailwaysStationCode()
     return true;
 }
 
+bool TrainStationDbGenerator::fetchFinishStationCodes()
+{
+    const auto stationArray = WikiData::query(R"(
+        SELECT DISTINCT ?station ?stationLabel ?code ?coord ?ref WHERE {
+            ?station (wdt:P31/wdt:P279*) wd:Q55488.
+            ?station p:P296 ?codeStmt.
+            ?codeStmt ps:P296 ?code.
+            ?codeStmt prov:wasDerivedFrom ?refnode.
+            ?refnode pr:P854 ?ref.
+            OPTIONAL { ?station wdt:P625 ?coord. }
+            SERVICE wikibase:label { bd:serviceParam wikibase:language "en". }
+        } ORDER BY (?station))", "wikidata_trainstation_vrfi.json");
+    if (stationArray.isEmpty()) {
+        qWarning() << "Empty query result!";
+        return false;
+    }
+
+    for (const auto &stationData : stationArray) {
+        const auto stationObj = stationData.toObject();
+        const auto ref = stationObj.value(QLatin1String("ref")).toObject().value(QLatin1String("value")).toString();
+        if (!ref.contains(QLatin1String("rata.digitraffic.fi"), Qt::CaseInsensitive)) {
+            continue;
+        }
+        const auto uri = insertOrMerge(stationObj);
+
+        // TODO this filters 'Ä' and 'Ö' too, which seem to occur in a few cases?
+        const auto id = stationObj.value(QLatin1String("code")).toObject().value(QLatin1String("value")).toString().toUpper();
+        if (id.size() < 2 || id.size() > 4 || !Util::containsOnlyLetters(id)) {
+            ++m_idFormatViolations;
+            qWarning() << "VR (Finland) station id format violation" << id << uri;
+            continue;
+        }
+
+        const auto it = m_vrfiMap.find(id);
+        if (it != m_vrfiMap.end() && (*it).second != uri) {
+            ++m_idConflicts;
+            qWarning() << "Conflict on VR (Finland) station code" << id << uri << m_vrfiMap[id];
+        } else {
+            m_vrfiMap[id] = uri;
+        }
+    }
+
+    return true;
+}
+
 bool TrainStationDbGenerator::fetchCountryInformation()
 {
     const auto stationArray = WikiData::query(R"(
@@ -246,6 +279,7 @@ bool TrainStationDbGenerator::fetchCountryInformation()
 
     for (const auto &stationData : stationArray) {
         const auto uri = insertOrMerge(stationData.toObject(), true);
+        Q_UNUSED(uri)
     }
 
     return true;
@@ -303,13 +337,7 @@ void TrainStationDbGenerator::processStations()
             qDebug() << "Station has no geo coordinates:" << (*it).name << (*it).uri;
         }
 
-        (*it).tz = m_tzDb.timezoneForLocation((*it).isoCode, (*it).coord);
-        if ((*it).tz.isEmpty() && ((*it).coord.isValid() || !(*it).isoCode.isEmpty())) {
-            ++m_timezoneLookupFailure;
-            qWarning() << "Timezone lookup failure:" << (*it).name << (*it).uri;
-        }
-
-        if (!(*it).coord.isValid() && (*it).tz.isEmpty() && (*it).isoCode.isEmpty()) { // no useful information
+        if (!(*it).coord.isValid() && (*it).isoCode.isEmpty()) { // no useful information
             it = m_stations.erase(it);
         } else {
             ++it;
@@ -324,8 +352,6 @@ void TrainStationDbGenerator::writeStationData(QIODevice *out)
         out->write("    {");
         CodeGen::writeCoordinate(out, station.coord);
         out->write(", ");
-        CodeGen::writeTimezone(out, station.tz);
-        out->write(", ");
         CodeGen::writeCountryIsoCode(out, station.isoCode);
         out->write("}, // ");
         out->write(station.name.toUtf8());
@@ -372,15 +398,15 @@ void TrainStationDbGenerator::writeUICMap(QIODevice* out)
     out->write("};\n\n");
 }
 
-void TrainStationDbGenerator::writeGareConnexionMap(QIODevice *out)
+void TrainStationDbGenerator::writeSncfMap(QIODevice *out)
 {
-    out->write("static constexpr const TrainStationIdIndex<GaresConnexionsId> garesConnexionsId_table[] = {\n");
-    for (const auto &it : m_garesConnexionsIdMap) {
+    out->write("static constexpr const TrainStationIdIndex<SncfStationId> sncfStationId_table[] = {\n");
+    for (const auto &it : m_sncfIdMap) {
         const auto station = std::lower_bound(m_stations.begin(), m_stations.end(), it.second);
         if (station == m_stations.end() || (*station).uri != it.second) {
             continue;
         }
-        out->write("    { GaresConnexionsId{\"");
+        out->write("    { SncfStationId{\"");
         out->write(it.first.toUtf8());
         out->write("\"}, TrainStationIndex{");
         out->write(QByteArray::number((int)std::distance(m_stations.begin(), station)));
@@ -395,7 +421,7 @@ void TrainStationDbGenerator::writeIndianRailwaysMap(QIODevice *out)
 {
     // variable length identifiers, so we need a string table
     std::vector<uint16_t> offsets;
-    offsets.reserve(m_garesConnexionsIdMap.size());
+    offsets.reserve(m_sncfIdMap.size());
     uint16_t offset = 0;
 
     out->write("static constexpr const char indianRailwaysSationCode_stringtable[] =\n");
@@ -440,16 +466,38 @@ R"(static constexpr const struct {
     out->write("};\n\n");
 }
 
+void TrainStationDbGenerator::writeVRMap(QIODevice *out)
+{
+    out->write("static constexpr const TrainStationIdIndex<VRStationCode> vrfiConnexionsId_table[] = {\n");
+    for (const auto &it : m_vrfiMap) {
+        const auto station = std::lower_bound(m_stations.begin(), m_stations.end(), it.second);
+        if (station == m_stations.end() || (*station).uri != it.second) {
+            continue;
+        }
+        out->write("    { VRStationCode{\"");
+        out->write(it.first.toUtf8());
+        for (int i = 0; i < 4 - it.first.size(); ++i) {
+            out->write("\\0");
+        }
+        out->write("\"}, TrainStationIndex{");
+        out->write(QByteArray::number((int)std::distance(m_stations.begin(), station)));
+        out->write("} }, // ");
+        out->write((*station).name.toUtf8());
+        out->write("\n");
+    }
+    out->write("};\n\n");
+}
+
 void TrainStationDbGenerator::printSummary()
 {
     qDebug() << "Generated database containing" << m_stations.size() << "train stations";
     qDebug() << "IBNR index:" << m_ibnrMap.size() << "elements";
     qDebug() << "UIC index:" << m_uicMap.size() << "elements";
-    qDebug() << "Gares & Connexions ID index:" << m_garesConnexionsIdMap.size() << "elements";
+    qDebug() << "SNCF station code index:" << m_sncfIdMap.size() << "elements";
     qDebug() << "Indian Railwaiys station code index:" << m_indianRailwaysMap.size() << "elements";
+    qDebug() << "VR (Finland) station code index:" << m_vrfiMap.size() << "elements";
     qDebug() << "Identifier collisions:" << m_idConflicts;
     qDebug() << "Identifier format violations:" << m_idFormatViolations;
     qDebug() << "Coordinate conflicts:" << m_coordinateConflicts;
     qDebug() << "Country ISO code conflicts: " << m_countryConflicts;
-    qDebug() << "Failed timezone lookups:" << m_timezoneLookupFailure;
 }
